#ifndef _AGNIX_CHIPS_DMA_H_
#define _AGNIX_CHIPS_DMA_H_

#include <agnix/adi/adi.h>
#include <asm/types.h>

#define DMA_MAX_CHANNEL			7

#define DMA_START_PAGES			2
#define DMA_MAX_PAGES			32

#define DMA1_BASE_ADDR			0x00
#define DMA2_BASE_ADDR			0xC0

#define DMA1_REG_CMD			(DMA1_BASE_ADDR + 0x08)
#define DMA1_REG_STAT			(DMA1_BASE_ADDR + 0x08)
#define DMA1_REG_REQ			(DMA1_BASE_ADDR + 0x09)
#define DMA1_REG_MASK			(DMA1_BASE_ADDR + 0x0a)
#define DMA1_REG_MODE			(DMA1_BASE_ADDR + 0x0b)
#define DMA1_REG_FF			(DMA1_BASE_ADDR + 0x0c)
#define DMA1_REG_TEMP			(DMA1_BASE_ADDR + 0x0d)
#define DMA1_REG_RESET			(DMA1_BASE_ADDR + 0x0d)
#define DMA1_REG_CLR_MASK		(DMA1_BASE_ADDR + 0x0e)
#define DMA1_REG_MASK_ALL		(DMA1_BASE_ADDR + 0x0f)

#define DMA2_REG_CMD			(DMA2_BASE_ADDR + 0x10)
#define DMA2_REG_STAT			(DMA2_BASE_ADDR + 0x10)
#define DMA2_REG_REQ			(DMA2_BASE_ADDR + 0x12)
#define DMA2_REG_MASK			(DMA2_BASE_ADDR + 0x14)
#define DMA2_REG_MODE			(DMA2_BASE_ADDR + 0x16)
#define DMA2_REG_FF			(DMA2_BASE_ADDR + 0x18)
#define DMA2_REG_TEMP			(DMA2_BASE_ADDR + 0x1a)
#define DMA2_REG_RESET			(DMA2_BASE_ADDR + 0x1a)
#define DMA2_REG_CLR_MASK		(DMA2_BASE_ADDR + 0x1c)
#define DMA2_REG_MASK_ALL		(DMA2_BASE_ADDR + 0x1e)

#define DMA_ADDR_0			(DMA1_BASE_ADDR + 0x00) /* 8 bit DMA */
#define DMA_ADDR_1			(DMA1_BASE_ADDR + 0x02)
#define DMA_ADDR_2			(DMA1_BASE_ADDR + 0x04)
#define DMA_ADDR_3			(DMA1_BASE_ADDR + 0x06)
#define DMA_ADDR_4			(DMA1_BASE_ADDR + 0x00) /* 16 bit DMA */
#define DMA_ADDR_5			(DMA1_BASE_ADDR + 0x04)
#define DMA_ADDR_6			(DMA1_BASE_ADDR + 0x08)
#define DMA_ADDR_7			(DMA1_BASE_ADDR + 0x0c)

#define DMA_CNT_0			(DMA1_BASE_ADDR + 0x01) /* 8 bit DMA */
#define DMA_CNT_1			(DMA1_BASE_ADDR + 0x03)
#define DMA_CNT_2			(DMA1_BASE_ADDR + 0x05)
#define DMA_CNT_3			(DMA1_BASE_ADDR + 0x07)
#define DMA_CNT_4			(DMA1_BASE_ADDR + 0x12) /* 16 bit DMA */
#define DMA_CNT_5			(DMA1_BASE_ADDR + 0x16)
#define DMA_CNT_6			(DMA1_BASE_ADDR + 0x1a)
#define DMA_CNT_7			(DMA1_BASE_ADDR + 0x1e)

#define DMA_PAGE_0			0x87
#define DMA_PAGE_1			0x83
#define DMA_PAGE_2			0x81
#define DMA_PAGE_3			0x82
#define DMA_PAGE_5			0x8b
#define DMA_PAGE_6			0x89
#define DMA_PAGE_7			0x8a

#define DMA_MODE_READ			0x44
#define DMA_MODE_WRITE			0x48

#define dma_parm(chip_ptr)		((struct chip_dma_parm_s *)((chip_ptr)->internal_parm))
#define dma_ops(chip_ptr)		((struct chip_dma_ops_s *)((chip_ptr)->internal_ops))

struct chip_dma_ops_s {
    int (*dma_lock)           (struct chip_s *chip);
    int (*dma_unlock)         (struct chip_s *chip);
    int (*dma_request_channel)(struct chip_s *chip, u8 dma_nr);
    int (*dma_put_channel)    (struct chip_s *chip, u8 dma_nr);
    int (*dma_enable)         (struct chip_s *chip, u8 dma_nr);
    int (*dma_disable)        (struct chip_s *chip, u8 dma_nr);
    int (*dma_clear_ff)       (struct chip_s *chip, u8 dma_nr);
    int (*dma_set_addr)       (struct chip_s *chip, u8 dma_nr, u32 address);
    int (*dma_set_mode)       (struct chip_s *chip, u8 dma_nr, u8 mode);
    int (*dma_set_count)      (struct chip_s *chip, u8 dma_nr, u16 count);
};

struct chip_pit_parm_s {
    u32	dma_address;
    u8  dma_mode;
    u16 dma_count;
};

int dma_init(void);

#endif

