#ifndef _AGNIX_TERMINAL_H_
#define _AGNIX_TERMINAL_H_

#include <agnix/wait.h>
#include <agnix/spinlock.h>
#include <agnix/list.h>
#include <agnix/syscons.h>

#define MAX_TERMINALS		16

struct terminal_s;

struct terminal_buf_s {
    int head;
    int tail;
    int data_len;
    char *data;
    
    int input_head;
    int input_tail;
    int input_data_len;
    char *input;
};

struct terminal_ops_s {
    int (* init)(struct terminal_s *term);
    int (* read)(struct terminal_s *term, char *buf, int count);    
    int (* write)(struct terminal_s *term, char *buf, int count);    
    int (* write_char)(struct terminal_s *term, char character);    
    int (* reset)(struct terminal_s *term);
};

struct terminal_var_s {
    int term_size_x;
    int term_size_y;
};

struct terminal_cmap_s {
    int term_cur_fgc;
    int term_cur_bgc;
};

struct terminal_driver_ops_s {
    int (* init)(void);
    int (* refresh)(struct terminal_s *term, int screen_scroll);
};

struct terminal_driver_s {
    struct list_head term_driver_list;
    struct terminal_driver_ops_s *ops;
    struct display_s *display;
};

struct terminal_s {
    const char *term_name;
    int term_nr;
    struct list_head term_list;
    struct list_head term_sys_list;
    struct terminal_buf_s term_buf;

    struct terminal_ops_s *ops;
    struct terminal_var_s *term_var;
    struct terminal_driver_s *term_driver;
    
    int term_active;
    int term_pgrp;

    struct wait_list_s wait_list;
};

int terminal_switch(int term_new);
int register_terminal(struct terminal_s *term);
int unregister_terminal(int term_nr);
int register_terminal_driver(struct terminal_s *term, struct terminal_driver_s *term_driver);
int terminals_init(void);

#endif

