/*
 * kernel_drivers/adi_char/terminal.c
 *
 * Copyright (c) 2003-2004 Lukasz Dembinski <dembol@nasa.com.pl>
 * All Rights Reserved
 * 
 * Date:	2004/01
 * Author: 	Lukasz Dembinski
 * Info:	terminal.c core file
 * Contact:	mailto: <dembol@nasa.com.pl>
 *
 */

#include <agnix/agnix.h>
#include <agnix/adi/adi.h>
#include <agnix/fs.h>
#include <agnix/terminal.h>
#include <agnix/resources.h>
#include <agnix/list.h>
#include <agnix/spinlock.h>
#include <agnix/sprintf.h>
#include <agnix/console.h>

#define MOD_NAME		"TERM: "

LIST_HEAD(terminal_list);
LIST_HEAD(terminal_sys_list);
spinlock_t terminal_list_lock;

extern struct terminal_s term_sys[MAX_TERMINALS];
extern int term_cur;
int term_nr = 0;

int terminal_switch(int term_switch)
{
    struct terminal_s *term = &term_sys[term_cur];
    struct terminal_s *term_new = &term_sys[term_switch];
    
    if (term_new->term_driver && term_new->term_driver->ops->refresh) {
    	term_cur 		= term_switch;
	term_new->term_driver->ops->refresh(term_new, 0);
	term->term_active 	= 0;
	term_new->term_active 	= 1;
    }
    
    return 0;    
}

int terminal_open(struct file_s *file, int flag, int mode)
{
    int tty_nr = (int)file->file_data;

    if (term_sys[tty_nr].term_pgrp != 0)
	return -1;

    term_sys[tty_nr].term_pgrp = current_task->t_pid;
    current_task->term    = &term_sys[tty_nr];
    file->file_data       = (void *)&term_sys[tty_nr];
    
    return 0;
}

int terminal_read(struct file_s *file, char *buf, int count)
{
    struct terminal_s *term = (struct terminal_s *)file->file_data;

    if (term == NULL)
	return -1;

    return syscons_read(term, buf, count);
}

int terminal_write(struct file_s *file, char *buf, int count)
{
    struct terminal_s *term = (struct terminal_s *)file->file_data;

    if (term == NULL)
	return -1;

    return syscons_write(term, buf, count);
}

int put_free_terminal(int terminal_desc)
{
    return 0;
}

int get_free_terminal(void)
{
    return term_nr++;
}

int set_terminal_var(int term_nr, struct terminal_var_s *term_var)
{
    return 0;    
}

struct file_ops_s term_file_ops = {
    .open      = terminal_open,
    .read      = terminal_read,
    .write     = terminal_write,
};

struct adi_chrdev_s term_chrdev = {
    .file_ops = &term_file_ops,
};

int register_terminal(struct terminal_s *term)
{
    term->term_nr = get_free_terminal();

    if (term->ops && term->ops->init)
	term->ops->init(term);

    list_add(&term->term_list, &terminal_list);

    sprintf(term_chrdev.dev_name, "tty%d", term->term_nr);
    term_chrdev.dev_nr = (DEV_TTY_MAJOR << 8) | (u8)term->term_nr;
    adi_register_chrdev(&term_chrdev);

    return term->term_nr;
}

int register_sys_terminal(struct terminal_s *term)
{
    spin_lock(&terminal_list_lock);
    list_add(&term->term_list, &terminal_sys_list);
    spin_unlock(&terminal_list_lock);

    return 0;
}

int register_terminal_driver(struct terminal_s *term, struct terminal_driver_s *term_driver)
{
    term->term_driver = term_driver;
    
    if (term_driver->ops && term_driver->ops->init)
	term_driver->ops->init();

    return 0;
}

int terminals_init(void)
{
    spin_lock_init(&terminal_list_lock);
    
    return 0;
}
