/*
 * kernel_sys/file.c
 *
 * Copyright (c) 2003-2005 Lukasz Dembinski <dembol@nasa.com.pl>
 * All Rights Reserved
 * 
 * Date:	2005/04
 * Author: 	Lukasz Dembinski
 * Info:	It's only stub for real virtual filesystem
 * Contact:	mailto: <dembol@nasa.com.pl>
 *
 */

#include <agnix/agnix.h>
#include <agnix/fs.h>
#include <agnix/terminal.h>
#include <agnix/memory.h>
#include <agnix/console.h>

#define MOD_NAME	"VFS: "

int file_install_fd(struct file_s *file)
{
    int fd;
    
    if ((fd = get_free_resource(current_task->files->fd_resource_desc)) < 0)
	return -1;

    if (fd > MAX_FILES)
	return -1;

    current_task->files->file[fd] = file;
    
    return fd;
}

struct file_s *file_create(void)
{
    struct file_s *file;
    
    file = (struct file_s *)get_free_pages(0);
    
    return file;
}

void file_destroy(struct file_s *file)
{
    put_free_pages((u32)file, 0);
}

struct file_s **file_table_create(void)
{
    struct file_s **file;
    
    file = (struct file_s **)get_free_pages(0);
    
    return file;
}

struct files_s *files_create(void)
{
    struct files_s *files;

    files = (struct files_s *)get_free_pages(0);

    files->fd_resource.resource_name   = "fds";
    files->fd_resource.resource_bitmap = files->fd;
    files->fd_resource.resource_len    = FILES_BITMAP_LEN;
    files->file			       = file_table_create();
    
    files->fd_resource_desc = register_resource(&files->fd_resource);

    return files;
}

struct files_s *files_copy(struct files_s *cur_files)
{
    struct files_s *files;
    
    files = files_create();
    
    memcpy(files->fd, cur_files->fd, FILES_BITMAP_LEN);

    return files;
}

int files_fork(struct task_s *cur_task, struct task_s *new_task)
{
    if (cur_task->files == NULL) {
	new_task->files = files_create();
    } else {
	new_task->files = files_copy(cur_task->files);
    }
    
    return 0;
}
