/*
 * kernel_net/layer_2/core.c
 *
 * Copyright (c) 2003-2005 Lukasz Dembinski <dembol@nasa.com.pl>
 * All Rights Reserved
 * 
 * Date:	2005/06
 * Author: 	Lukasz Dembinski
 * Info:	core.c core file
 * Contact:	mailto: <dembol@nasa.com.pl>
 *
 */

#include <agnix/agnix.h>
#include <agnix/adi/adi.h>
#include <agnix/memory.h>
#include <agnix/net/net.h>
#include <agnix/list.h>
#include <agnix/console.h>

#define MOD_NAME	"NET: "

struct list_head 	layer_2_pack_list;
spinlock_t 		layer_2_pack_list_lock;

int net_layer_2_add_pack(struct net_layer_2_pack *pack)
{
    u32 flags;

    spin_lock_irqsave(&layer_2_pack_list_lock, flags);
    list_add(&pack->pack_list, &layer_2_pack_list);
    spin_unlock_irqrestore(&layer_2_pack_list_lock, flags);

    return 0;
}

int net_layer_2_recv_pack(struct frb_s *frb)
{
    u32 flags;
//    struct frb_s *frb_clone;
    struct list_head *tmp;
    struct net_layer_2_pack *pack;

    frb->frb_dev->if_ops->header_parse(frb);
    spin_lock_irqsave(&layer_2_pack_list_lock, flags);

    if (!list_empty(&layer_2_pack_list)) {

        list_for_each(tmp, &layer_2_pack_list) {
	    pack = list_entry(tmp, struct net_layer_2_pack, pack_list);

	    if (pack->pack_type != frb->frb_proto_type)
		continue;
	
//	frb_clone = frb;
//	if (tmp->next != &layer_2_pack_list) {
//	    frb_clone = (struct frb_s *)get_free_pages(0);
//	    memcpy((void *)frb_clone, (void *)frb, sizeof(struct frb_s));
//	}

	    pack->receiver(frb);
	}
    }    
    
    spin_unlock_irqrestore(&layer_2_pack_list_lock, flags);
    
    return 0;
}

int net_layer_2_init(void)
{
    printk(MOD_NAME "\tLayer 2: core");

    INIT_LIST_HEAD(&layer_2_pack_list);
    spin_lock_init(&layer_2_pack_list_lock);

    net_layer_2_eth_init();
    net_layer_2_arp_init();

    printk("\n");

    return 0;
}
