/*
 * kernel_net/layer_2/eth/eth_core.c
 *
 * Copyright (c) 2003-2005 Lukasz Dembinski <dembol@nasa.com.pl>
 * All Rights Reserved
 * 
 * Date:	2005/06
 * Author: 	Lukasz Dembinski
 * Info:	eth_core.c core file
 * Contact:	mailto: <dembol@nasa.com.pl>
 *
 */

#include <agnix/agnix.h>
#include <agnix/adi/adi.h>
#include <agnix/init.h>
#include <agnix/net/net.h>
#include <agnix/memory.h>
#include <agnix/console.h>
#include <agnix/sprintf.h>

#define ETH_CORE_DEBUG	0
#define MOD_NAME	"NET: "

int eth_device_nr = 0;

int eth_header_build(struct frb_s *frb, u8 *mac_dest, u8 *mac_source, u16 proto)
{
    struct eth_hdr_s *eth_hdr;
    
    eth_hdr = (struct eth_hdr_s *)frb_pop(frb, sizeof(struct eth_hdr_s));
    memcpy((void *)eth_hdr->mac_dest, mac_dest, NET_LAYER_2_ETH_MAC_LEN);

    if (mac_source != NULL)
	memcpy((void *)eth_hdr->mac_source, mac_source, NET_LAYER_2_ETH_MAC_LEN);
    else
	memcpy((void *)eth_hdr->mac_source, frb->frb_dev->if_layer_2.if_addr, NET_LAYER_2_ETH_MAC_LEN);
	
    eth_hdr->proto = htons(proto);

    return 0;
}

int eth_header_parse(struct frb_s *frb)
{
    struct eth_hdr_s *ethhdr = (struct eth_hdr_s *)frb_data(frb);
#if ETH_CORE_DEBUG
    int i;

    u8 *ptr = (u8 *)frb->frb_data;

    for (i = 0; i < 64; i++) {
	printk("%02X ", ptr[i]);
    }
    printk("\n");

#endif

    if (ntohs(ethhdr->proto) > 1536)
	frb->frb_proto_type = ethhdr->proto;
    else
	frb->frb_proto_type = ETH_P_802_3;
    
    if (frb->frb_data[0] & 0x01) {
	if (!memcmp(frb->frb_data, frb->frb_dev->if_layer_2.if_addr_broadcast, NET_LAYER_2_ETH_MAC_LEN))
	    frb->frb_dest_type = FRB_DEST_TYPE_BROADCAST;
    } else
    if (!memcmp(frb->frb_data, frb->frb_dev->if_layer_2.if_addr, NET_LAYER_2_ETH_MAC_LEN))
	frb->frb_dest_type = FRB_DEST_TYPE_LOCAL;
    else
	frb->frb_dest_type = FRB_DEST_TYPE_OTHER;

    frb_push(frb, sizeof(struct eth_hdr_s));

    return 0;
}

int eth_set_mtu(struct adi_netdev_s *netdev, int mtu)
{
    return 0;
}

int eth_set_mac_addr(struct adi_netdev_s *netdev, u8 *mac_addr)
{
    return 0;
}

int eth_set_broadcast_addr(struct adi_netdev_s *netdev, u8 *broadcast_addr)
{
    memcpy(netdev->if_layer_2.if_addr_broadcast, broadcast_addr, NET_LAYER_2_ETH_MAC_LEN);

    return 0;
}

int eth_device_compose_name(struct adi_netdev_s *netdev)
{
    sprintf(netdev->if_name, "eth%d", eth_device_nr);    
    eth_device_nr++;

    return 0;
}

int eth_device_init(struct adi_netdev_s *netdev)
{
    netdev->if_layer_2.if_addr_len	= NET_LAYER_2_ETH_MAC_LEN;
    netdev->if_layer_2.if_mtu		= NET_LAYER_2_ETH_MTU;
    netdev->if_layer_2.if_type		= IF_TYPE_ETH;
    netdev->if_ops->set_mac_addr 	= eth_set_mac_addr;
    netdev->if_ops->set_broadcast_addr 	= eth_set_broadcast_addr;
    netdev->if_ops->set_mtu	 	= eth_set_mtu;
    netdev->if_ops->header_build	= eth_header_build;
    netdev->if_ops->header_parse 	= eth_header_parse;

    eth_set_broadcast_addr(netdev, NET_LAYER_2_ETH_BROADCAST);
    
    return 0;
}

int net_layer_2_eth_init(void)
{
    printk(", eth");
    
    return 0;
}
