/*
 * kernel_net/net_core/frb.c
 *
 * Copyright (c) 2003-2005 Lukasz Dembinski <dembol@nasa.com.pl>
 * All Rights Reserved
 * 
 * Date:	2005/06
 * Author: 	Lukasz Dembinski
 * Info:	frb.c core file
 * Contact:	mailto: <dembol@nasa.com.pl>
 *
 */

#include <agnix/agnix.h>
#include <agnix/net/net.h>
#include <agnix/memory.h>

#define MOD_NAME	"NET: "

u8 *frb_data(struct frb_s *frb)
{
    return (frb->frb_data + frb->frb_offset);
}

u8 *frb_push(struct frb_s *frb, int size)
{
    frb->frb_offset += size;
    
    return frb_data(frb);
}

u8 *frb_pop(struct frb_s *frb, int size)
{
    if (frb->frb_offset < size)
	return NULL;
	
    frb->frb_offset -= size;
    frb->frb_len    += size;
    
    return frb_data(frb);
}

int frb_create(struct frb_s *frb)
{
    frb->frb_len     = 0;
    frb->frb_offset  = 0;
    frb->frb_data    = (u8 *)get_free_pages(0);
    
    return 0;
}

int frb_destroy(struct frb_s *frb)
{
    put_free_pages((u32)frb->frb_data, 0);
    
    return 0;
}

int frb_reverse(struct frb_s *frb)
{
    frb->frb_offset = 1000;
    
    return 0;
}
