/*
 * Copyright (c) 2003-2004 Lukasz Dembinski <dembol@nasa.com.pl>
 * All Rights Reserved
 * 
 * Date:	2004/01
 * Author: 	Lukasz Dembinski
 * Info:	init.c file
 * Contact:	mailto: <dembol@nasa.com.pl>
 *
 */


#include <agnix/agnix.h>
#include <asm/core.h>
#include <agnix/config.h>
#include <agnix/sched.h>
#include <agnix/console.h>
#include <agnix/ioport.h>
#include <agnix/adi/adi.h>
#include <agnix/net/net.h>
#include <agnix/crypt.h>
#include <agnix/syscalls.h>
#include <agnix/threads.h>
#include <agnix/tasks.h>
#include <agnix/queues.h>
#include <agnix/memory.h>
#include <agnix/resources.h>
#include <agnix/data_structures.h>
#include <asm/memtests.h>
#include <agnix/counters.h>
#include <agnix/timers.h>
#include <agnix/terminal.h>
#include <agnix/pm.h>
#include <agnix/fastirq.h>
#include <agnix/agnix_console.h>

int errno;
extern char *agnix_banner;
extern char *agnix_version;
extern char *agnix_license;
extern char _text;
extern char _end;

void init_thread(void *data)
{
    /* TODO */
}

void init_start(void)
{
//    create_kernel_thread("init", init_thread, (void *)2);
}

int clean_kernel(void)
{
    printk("Cleaning kernel ...\n");
    
    bootmem_free_init();

    return 0;
}

void start_kernel(void)
{
    terminals_init();

#if CONFIG_DRIVERS_CHAR
    adi_console_init();
#endif

    memalloc_init();
    resources_init();
    counters_init();
    data_structures_init();
    queues_init();
    adi_chips_init();
        
    i386_init();
#if CONFIG_MEMTESTS
//    do_memtests();
#endif
    network_init();
    adi_drivers_init();
    fork_init();
    tasks_init();
    scheduler_init();

    __sti();
    cpu_calibrate_delay();
    __cli();

    fastirq_init();
    timers_init();
    crypt_init();

    init_start();
    pm_init();
    clean_kernel();

    printk(NEW_LINE);
    printk(agnix_banner);
    printk(agnix_version);
    printk(agnix_license);
    printk(NEW_LINE);

    agnix_console_start();

    __sti();

    for(;;);
}
