/*
 * kernel_super/sched/wait.c
 *
 * Copyright (c) 2003-2004 Lukasz Dembinski <dembol@nasa.com.pl>
 * All Rights Reserved
 * 
 * Date:	2004/01
 * Author: 	Lukasz Dembinski
 * Info:	wait.c core file
 * Contact:	mailto: <dembol@nasa.com.pl>
 *
 */

#include <agnix/agnix.h>
#include <agnix/list.h>
#include <agnix/tasks.h> 
#include <agnix/sched.h> 
#include <agnix/spinlock.h>
#include <agnix/wait.h>

void wait_list_add(struct task_s *task, struct wait_list_s *waitlist)
{
    u32 flags;

    spin_lock_irqsave(&(waitlist->lock), flags);
    list_add(&(task->wait_list), &(waitlist->list));
    spin_unlock_irqrestore(&(waitlist->lock), flags);
}

void wait_list_del(struct task_s *task, struct wait_list_s *waitlist)
{
    u32 flags;

    spin_lock_irqsave(&(waitlist->lock), flags);
    list_del(&(task->wait_list));
    spin_unlock_irqrestore(&(waitlist->lock), flags);
}

void wait_list_sleep_on(struct task_s *task, struct wait_list_s *waitlist)
{
    wait_list_add(task, waitlist);    
    sched_deactivate_task(task);
    schedule_task();
}

void wait_list_remove(struct task_s *task, struct wait_list_s *waitlist)
{
    wait_list_del(task, waitlist);    
}

void wait_list_wake_up(struct wait_list_s *waitlist)
{
    struct list_head *tmp;
    struct task_s *task;
    u32 flags;

    spin_lock_irqsave(&waitlist->lock, flags);
    
    if (list_empty(&waitlist->list)) {
        spin_unlock_irqrestore(&waitlist->lock, flags);
	return;
    }

    list_for_each(tmp, &waitlist->list) {
	task = list_entry(tmp, struct task_s, wait_list);
	sched_activate_task(task);
    }
    
    INIT_LIST_HEAD(&waitlist->list);
    
    spin_unlock_irqrestore(&waitlist->lock, flags);
}

void wait_list_init(struct wait_list_s *waitlist)
{
    INIT_LIST_HEAD(&waitlist->list);
    spin_lock_init(&waitlist->lock);
}
